@@ -74,15 +74,15 @@ static grub_ssize_t
 read_vli (grub_file_t file, grub_uint64_t *num)
 {
   grub_uint8_t buf[VLI_MAX_DIGITS];
-  grub_ssize_t read;
+  grub_ssize_t read_bytes;
   grub_size_t dec;
 
-  read = grub_file_read (file, buf, VLI_MAX_DIGITS);
-  if (read < 0)
+  read_bytes = grub_file_read (file, buf, VLI_MAX_DIGITS);
+  if (read_bytes < 0)
     return -1;
 
-  dec = decode_vli (buf, read, num);
-  grub_file_seek (file, file->offset - (read - dec));
+  dec = decode_vli (buf, read_bytes, num);
+  grub_file_seek (file, file->offset - (read_bytes - dec));
   return dec;
 }
 
@@ -169,12 +169,14 @@ ERROR:
 }
 
 static grub_file_t
-grub_xzio_open (grub_file_t io,
-		const char *name __attribute__ ((unused)))
+grub_xzio_open (grub_file_t io, enum grub_file_type type)
 {
   grub_file_t file;
   grub_xzio_t xzio;
 
+  if (type & GRUB_FILE_TYPE_NO_DECOMPRESS)
+    return io;
+
   file = (grub_file_t) grub_zalloc (sizeof (*file));
   if (!file)
     return 0;
@@ -281,7 +283,7 @@ grub_xzio_read (grub_file_t file, char *buf, grub_size_t len)
 
       {
 	grub_off_t new_offset = current_offset + xzio->buf.out_pos;
-	
+
 	if (file->offset <= new_offset)
 	  /* Store first chunk of data in buffer.  */
 	  {
@@ -319,16 +321,17 @@ grub_xzio_close (grub_file_t file)
 
   /* Device must not be closed twice.  */
   file->device = 0;
+  file->name = 0;
   return grub_errno;
 }
 
 static struct grub_fs grub_xzio_fs = {
   .name = "xzio",
-  .dir = 0,
-  .open = 0,
-  .read = grub_xzio_read,
-  .close = grub_xzio_close,
-  .label = 0,
+  .fs_dir = 0,
+  .fs_open = 0,
+  .fs_read = grub_xzio_read,
+  .fs_close = grub_xzio_close,
+  .fs_label = 0,
   .next = 0
 };
 
