@@ -28,9 +28,6 @@
 
 grub_partition_map_t grub_partition_map_list;
 
-#define MAX_RECURSION_DEPTH	32
-static unsigned int recursion_depth = 0;
-
 /*
  * Checks that disk->partition contains part.  This function assumes that the
  * start of part is relative to the start of disk->partition.  Returns 1 if
@@ -125,22 +122,14 @@ grub_partition_probe (struct grub_disk *disk, const char *str)
   for (ptr = str; *ptr;)
     {
       grub_partition_map_t partmap;
-      unsigned long num;
+      int num;
       const char *partname, *partname_end;
 
       partname = ptr;
       while (*ptr && grub_isalpha (*ptr))
 	ptr++;
       partname_end = ptr;
-
-      num = grub_strtoul (ptr, &ptr, 0);
-      if (num == 0 || num > GRUB_INT_MAX)
-	{
-	  grub_error (GRUB_ERR_BAD_NUMBER, N_("invalid partition number"));
-	  return 0;
-	}
-
-      num -= 1;
+      num = grub_strtoul (ptr, &ptr, 0) - 1;
 
       curpart = 0;
       /* Use the first partition map type found.  */
@@ -219,12 +208,7 @@ part_iterate (grub_disk_t dsk, const grub_partition_t partition, void *data)
       FOR_PARTITION_MAPS(partmap)
       {
 	grub_err_t err;
-	recursion_depth++;
-	if (recursion_depth <= MAX_RECURSION_DEPTH)
-	  err = partmap->iterate (dsk, part_iterate, ctx);
-	else
-	  err = grub_error (GRUB_ERR_RECURSION_DEPTH, "maximum recursion depth exceeded");
-	recursion_depth--;
+	err = partmap->iterate (dsk, part_iterate, ctx);
 	if (err)
 	  grub_errno = GRUB_ERR_NONE;
 	if (ctx->ret)
