@@ -27,6 +27,11 @@
 
 #include <sys/types.h>
 
+#define _gcry_mpi_invm gcry_mpi_invm
+#define _gcry_mpi_set gcry_mpi_set
+#define _gcry_mpi_set_ui gcry_mpi_set_ui
+#define size_t grub_size_t
+
 #undef __GNU_LIBRARY__
 #define __GNU_LIBRARY__ 1
 
@@ -46,21 +51,17 @@ selftest (void)
 }
 
 static inline int
-fips_mode (void)
+_gcry_fips_mode (void)
 {
   return 0;
 }
 
+#define assert gcry_assert
+
 #ifdef GRUB_UTIL
-#pragma GCC diagnostic ignored "-Wshadow"
 
-static inline void *
-memset (void *s, int c, grub_size_t n)
-{
-  return grub_memset (s, c, n);
-}
+#define memset grub_memset
 
-#pragma GCC diagnostic error "-Wshadow"
 #endif
 
 
