@@ -40,7 +40,7 @@ grub_appleloader_unload (void)
   grub_efi_boot_services_t *b;
 
   b = grub_efi_system_table->boot_services;
-  efi_call_1 (b->unload_image, image_handle);
+  b->unload_image (image_handle);
 
   grub_free (cmdline);
   cmdline = 0;
@@ -55,7 +55,7 @@ grub_appleloader_boot (void)
   grub_efi_boot_services_t *b;
 
   b = grub_efi_system_table->boot_services;
-  efi_call_3 (b->start_image, image_handle, 0, 0);
+  b->start_image (image_handle, 0, 0);
 
   grub_appleloader_unload ();
 
@@ -76,7 +76,7 @@ struct piwg_full_device_path
       .header = {							\
 	.type = GRUB_EFI_HARDWARE_DEVICE_PATH_TYPE,			\
 	.subtype = GRUB_EFI_MEMORY_MAPPED_DEVICE_PATH_SUBTYPE,		\
-	.length = {sizeof (struct grub_efi_memory_mapped_device_path), 0} \
+	.length = sizeof (struct grub_efi_memory_mapped_device_path)	\
       },								\
       .memory_type = GRUB_EFI_MEMORY_MAPPED_IO,				\
       .start_address = st,						\
@@ -87,16 +87,15 @@ struct piwg_full_device_path
 	.header = {							\
 	  .type = GRUB_EFI_MEDIA_DEVICE_PATH_TYPE,			\
 	  .subtype = GRUB_EFI_PIWG_DEVICE_PATH_SUBTYPE,			\
-	  .length = {sizeof (struct grub_efi_piwg_device_path), 0}	\
+	  .length = sizeof (struct grub_efi_piwg_device_path)		\
 	},								\
-	.guid = {0x2B0585EB, 0xD8B8, 0x49A9, {0x8B, 0x8C, 0xE2, 0x1B,	\
-					      0x01, 0xAE, 0xF2, 0xB7}}	\
+	.guid = GRUB_EFI_VENDOR_APPLE_GUID				\
       },								\
        .end =								\
 	  {								\
 	    .type = GRUB_EFI_END_DEVICE_PATH_TYPE,			\
 	    .subtype = GRUB_EFI_END_ENTIRE_DEVICE_PATH_SUBTYPE,		\
-	    .length = {sizeof (struct grub_efi_device_path), 0}		\
+	    .length = sizeof (struct grub_efi_device_path)		\
 	  }								\
   }
 
@@ -127,7 +126,7 @@ static struct piwg_full_device_path devpath_6 = MAKE_PIWG_PATH (0xffcc4000,
 static struct piwg_full_device_path devpath_7 = MAKE_PIWG_PATH (0xff981000,
 								0xffc8ffff);
 
-/* mid-2012 MBP retina (MacBookPro10,1) */ 
+/* mid-2012 MBP retina (MacBookPro10,1) */
 static struct piwg_full_device_path devpath_8 = MAKE_PIWG_PATH (0xff990000,
 								0xffb2ffff);
 
@@ -166,8 +165,8 @@ grub_cmd_appleloader (grub_command_t cmd __attribute__ ((unused)),
   b = grub_efi_system_table->boot_services;
 
   for (pdev = devs ; pdev->devpath ; pdev++)
-    if (efi_call_6 (b->load_image, 0, grub_efi_image_handle, pdev->devpath,
-                    NULL, 0, &image_handle) == GRUB_EFI_SUCCESS)
+    if (b->load_image (0, grub_efi_image_handle, pdev->devpath,
+                       NULL, 0, &image_handle) == GRUB_EFI_SUCCESS)
       break;
 
   if (! pdev->devpath)
