@@ -33,6 +33,7 @@
 #include <grub/mm.h>
 #include <grub/cpu/relocator.h>
 #include <grub/extcmd.h>
+#include <grub/verify.h>
 
 GRUB_MOD_LICENSE ("GPLv3+");
 
@@ -55,7 +56,7 @@ static const struct grub_arg_option options[] =
 	means that the GRUBDEVICE and PLAN9DEVICE are
 	actually the same device, just named differently
 	in OS and GRUB.  */
-     N_("Override guessed mapping of Plan9 devices."), 
+     N_("Override guessed mapping of Plan9 devices."),
      N_("GRUBDEVICE=PLAN9DEVICE"),
      ARG_TYPE_STRING},
     {0, 0, 0, 0, 0, 0}
@@ -77,7 +78,7 @@ struct grub_plan9_header
 static grub_err_t
 grub_plan9_boot (void)
 {
-  struct grub_relocator32_state state = { 
+  struct grub_relocator32_state state = {
     .eax = 0,
     .eip = eip,
     .ebx = 0,
@@ -121,7 +122,7 @@ static const char prefixes[5][10] = {
 
 #include <grub/err.h>
 
-static inline grub_err_t 
+static inline grub_err_t
 grub_extend_alloc (grub_size_t sz, grub_size_t *allocated, char **ptr)
 {
   void *n;
@@ -301,7 +302,7 @@ fill_disk (const char *name, void *data)
 	    grub_device_close (dev);
 	    return 0;
 	  }
-	
+
 	/* if it's the disk the kernel is loaded from we need to name
 	   it nevertheless.  */
 	plan9name = grub_strdup ("sdZ0");
@@ -330,7 +331,7 @@ fill_disk (const char *name, void *data)
 	    plan9name = grub_xasprintf ("sd%c%d", 'C' + unit / 2, unit % 2);
 	    break;
 	  }
-	
+
 	/* FIXME: how does Plan9 number controllers?
 	   We probably need save the SCSI devices and sort them  */
 	plan9name
@@ -413,7 +414,7 @@ grub_cmd_plan9 (grub_extcmd_context_t ctxt, int argc, char *argv[])
   if (!rel)
     goto fail;
 
-  fill_ctx.file = grub_file_open (argv[0]);
+  fill_ctx.file = grub_file_open (argv[0], GRUB_FILE_TYPE_PLAN9_KERNEL);
   if (! fill_ctx.file)
     goto fail;
 
@@ -505,6 +506,7 @@ grub_cmd_plan9 (grub_extcmd_context_t ctxt, int argc, char *argv[])
   configptr = grub_stpcpy (configptr, "bootfile=");
   configptr = grub_stpcpy (configptr, bootpath);
   *configptr++ = '\n';
+  char *cmdline = configptr;
   {
     int i;
     for (i = 1; i < argc; i++)
@@ -513,6 +515,15 @@ grub_cmd_plan9 (grub_extcmd_context_t ctxt, int argc, char *argv[])
 	*configptr++ = '\n';
       }
   }
+
+  {
+    grub_err_t err;
+    *configptr = '\0';
+    err = grub_verify_string (cmdline, GRUB_VERIFY_KERNEL_CMDLINE);
+    if (err)
+      goto fail;
+  }
+
   configptr = grub_stpcpy (configptr, fill_ctx.pmap);
 
   {
