@@ -28,14 +28,8 @@
 #include <grub/memory.h>
 #endif
 
-#define GRUB_MEMORY_MACHINE_SCRATCH_ADDR	0x68000
-#define GRUB_MEMORY_MACHINE_SCRATCH_SEG	(GRUB_MEMORY_MACHINE_SCRATCH_ADDR >> 4)
-#define GRUB_MEMORY_MACHINE_SCRATCH_SIZE	0x10000
-
-#define GRUB_MEMORY_MACHINE_LOWER_USABLE		0x9fc00		/* 640 kiB - 1 kiB */
-
-#define GRUB_MEMORY_MACHINE_UPPER_START			0x100000	/* 1 MiB */
-#define GRUB_MEMORY_MACHINE_LOWER_SIZE			GRUB_MEMORY_MACHINE_UPPER_START
+#include <grub/i386/memory.h>
+#include <grub/i386/memory_raw.h>
 
 #ifndef ASM_FILE
 
