@@ -107,22 +107,6 @@ struct section_metadata
   const char *strtab;
 };
 
-#define GRUB_SBAT_NOTE_NAME ".sbat"
-#define GRUB_SBAT_NOTE_TYPE 0x53424154 /* "SBAT" */
-
-struct grub_sbat_note {
-  Elf32_Nhdr header;
-  char name[ALIGN_UP(sizeof(GRUB_SBAT_NOTE_NAME), 4)];
-};
-
-#define GRUB_APPENDED_SIGNATURE_NOTE_NAME "Appended-Signature"
-#define GRUB_APPENDED_SIGNATURE_NOTE_TYPE 0x41536967 /* "ASig" */
-struct grub_appended_signature_note
-{
-  Elf32_Nhdr header;
-  char name[ALIGN_UP (sizeof (GRUB_APPENDED_SIGNATURE_NOTE_NAME), 4)];
-};
-
 static int
 is_relocatable (const struct grub_install_image_target_desc *image_target)
 {
@@ -224,7 +208,7 @@ grub_arm_reloc_jump24 (grub_uint32_t *target, Elf32_Addr sym_addr)
 
 void
 SUFFIX (grub_mkimage_generate_elf) (const struct grub_install_image_target_desc *image_target,
-				    int note, char *sbat, size_t appsig_size, char **core_img, size_t *core_size,
+				    int note, char **core_img, size_t *core_size,
 				    Elf_Addr target_addr,
 				    struct grub_mkimage_layout *layout)
 {
@@ -233,24 +217,10 @@ SUFFIX (grub_mkimage_generate_elf) (const struct grub_install_image_target_desc
   Elf_Ehdr *ehdr;
   Elf_Phdr *phdr;
   Elf_Shdr *shdr;
-  int header_size, footer_size = 0, footer_offset = 0;
+  int header_size, footer_size = 0;
   int phnum = 1;
   int shnum = 4;
   int string_size = sizeof (".text") + sizeof ("mods") + 1;
-  char *footer;
-
-  if (sbat)
-    {
-      phnum++;
-      footer_size += ALIGN_UP (sizeof (struct grub_sbat_note) + layout->sbat_size, 4);
-    }
-
-  if (appsig_size)
-    {
-      phnum++;
-      footer_size += ALIGN_UP (sizeof (struct grub_appended_signature_note), 4);
-      footer_size += ALIGN_UP_OVERHEAD (appsig_size, 4);
-    }
 
   if (image_target->id != IMAGE_LOONGSON_ELF)
     phnum += 2;
@@ -278,7 +248,6 @@ SUFFIX (grub_mkimage_generate_elf) (const struct grub_install_image_target_desc
   ehdr = (void *) elf_img;
   phdr = (void *) (elf_img + sizeof (*ehdr));
   shdr = (void *) (elf_img + sizeof (*ehdr) + phnum * sizeof (*phdr));
-  footer = elf_img + program_size + header_size;
   memcpy (ehdr->e_ident, ELFMAG, SELFMAG);
   ehdr->e_ident[EI_CLASS] = ELFCLASSXX;
   if (!image_target->bigendian)
@@ -451,8 +420,6 @@ SUFFIX (grub_mkimage_generate_elf) (const struct grub_install_image_target_desc
       phdr->p_filesz = grub_host_to_target32 (XEN_NOTE_SIZE);
       phdr->p_memsz = 0;
       phdr->p_offset = grub_host_to_target32 (header_size + program_size);
-      footer = ptr;
-      footer_offset = XEN_NOTE_SIZE;
     }
 
   if (image_target->id == IMAGE_XEN_PVH)
@@ -486,8 +453,6 @@ SUFFIX (grub_mkimage_generate_elf) (const struct grub_install_image_target_desc
       phdr->p_filesz = grub_host_to_target32 (XEN_PVH_NOTE_SIZE);
       phdr->p_memsz = 0;
       phdr->p_offset = grub_host_to_target32 (header_size + program_size);
-      footer = ptr;
-      footer_offset = XEN_PVH_NOTE_SIZE;
     }
 
   if (note)
@@ -518,54 +483,6 @@ SUFFIX (grub_mkimage_generate_elf) (const struct grub_install_image_target_desc
       phdr->p_filesz = grub_host_to_target32 (note_size);
       phdr->p_memsz = 0;
       phdr->p_offset = grub_host_to_target32 (header_size + program_size);
-      footer = (elf_img + program_size + header_size + note_size);
-      footer_offset += note_size;
-    }
-
-  if (sbat)
-    {
-      int note_size = ALIGN_UP (sizeof (struct grub_sbat_note) + layout->sbat_size, 4);
-      struct grub_sbat_note *note_ptr = (struct grub_sbat_note *) footer;
-
-      note_ptr->header.n_namesz = grub_host_to_target32 (sizeof (GRUB_SBAT_NOTE_NAME));
-      note_ptr->header.n_descsz = grub_host_to_target32 (ALIGN_UP(layout->sbat_size, 4));
-      note_ptr->header.n_type = grub_host_to_target32 (GRUB_SBAT_NOTE_TYPE);
-      memcpy (note_ptr->name, GRUB_SBAT_NOTE_NAME, sizeof (GRUB_SBAT_NOTE_NAME));
-      memcpy ((char *)(note_ptr + 1), sbat, layout->sbat_size);
-
-      phdr++;
-      phdr->p_type = grub_host_to_target32 (PT_NOTE);
-      phdr->p_flags = grub_host_to_target32 (PF_R);
-      phdr->p_align = grub_host_to_target32 (image_target->voidp_sizeof);
-      phdr->p_vaddr = 0;
-      phdr->p_paddr = 0;
-      phdr->p_filesz = grub_host_to_target32 (note_size);
-      phdr->p_memsz = 0;
-      phdr->p_offset = grub_host_to_target32 (header_size + program_size + footer_offset);
-      footer += note_size;
-      footer_offset += note_size;
-    }
-
-  if (appsig_size)
-    {
-      int note_size = ALIGN_UP (sizeof (struct grub_appended_signature_note) + appsig_size, 4);
-      struct grub_appended_signature_note *note_ptr = (struct grub_appended_signature_note *) footer;
-
-      note_ptr->header.n_namesz = grub_host_to_target32 (sizeof (GRUB_APPENDED_SIGNATURE_NOTE_NAME));
-      /* Needs to sit at the end, so we round this up and sign some zero padding. */
-      note_ptr->header.n_descsz = grub_host_to_target32 (ALIGN_UP (appsig_size, 4));
-      note_ptr->header.n_type = grub_host_to_target32 (GRUB_APPENDED_SIGNATURE_NOTE_TYPE);
-      strcpy (note_ptr->name, GRUB_APPENDED_SIGNATURE_NOTE_NAME);
-
-      phdr++;
-      phdr->p_type = grub_host_to_target32 (PT_NOTE);
-      phdr->p_flags = grub_host_to_target32 (PF_R);
-      phdr->p_align = grub_host_to_target32 (image_target->voidp_sizeof);
-      phdr->p_vaddr = 0;
-      phdr->p_paddr = 0;
-      phdr->p_filesz = grub_host_to_target32 (note_size);
-      phdr->p_memsz = 0;
-      phdr->p_offset = grub_host_to_target32 (header_size + program_size + footer_offset);
     }
 
   {
@@ -1393,7 +1310,7 @@ SUFFIX (relocate_addrs) (Elf_Ehdr *e, struct section_metadata *smd,
 		  */
 
 		 sym_addr += addend;
-		 off = (grub_int64_t) sym_addr - target_section_addr - offset - image_target->vaddr_offset;
+		 off = sym_addr - target_section_addr - offset - image_target->vaddr_offset;
 
 		 switch (ELF_R_TYPE (info))
 		   {
@@ -2514,8 +2431,6 @@ SUFFIX (grub_mkimage_load_image) (const char *kernel_path,
   if (!is_relocatable (image_target))
     {
       Elf_Addr current_address = layout->kernel_size;
-      Elf_Addr bss_start = layout->kernel_size;
-      bool is_first = true;
 
       for (i = 0, s = smd.sections;
 	   i < smd.num_sections;
@@ -2538,12 +2453,6 @@ SUFFIX (grub_mkimage_load_image) (const char *kernel_path,
 	      current_address = grub_host_to_target_addr (s->sh_addr)
 		- image_target->link_addr;
 
-	    if (is_first == true)
-	      {
-		bss_start = current_address;
-		is_first = false;
-	      }
-
 	    smd.vaddrs[i] = current_address
 	      + image_target->vaddr_offset;
 	    current_address += grub_host_to_target_addr (s->sh_size);
@@ -2551,16 +2460,6 @@ SUFFIX (grub_mkimage_load_image) (const char *kernel_path,
       current_address = ALIGN_UP (current_address + image_target->vaddr_offset,
 				  image_target->section_align)
 	- image_target->vaddr_offset;
-
-      if (image_target->id == IMAGE_YEELOONG_FLASH
-	  || image_target->id == IMAGE_FULOONG2F_FLASH
-	  || image_target->id == IMAGE_LOONGSON_ELF
-	  || image_target->id == IMAGE_QEMU_MIPS_FLASH
-	  || image_target->id == IMAGE_MIPS_ARC)
-	{
-	  layout->kernel_size = bss_start;
-	}
-
       layout->bss_size = current_address - layout->kernel_size;
     }
   else
